from typing import List

from fastapi import Depends, HTTPException, status

from baobaxia.permissions.model_permissions import ModelPermission
from baobaxia.permissions.roles import get_role_permissions
from baobaxia.saberes import Mocambola
from baobaxia.utils.mocambola import get_current_mocambola


class PermissionChecker:
    def __init__(self, permissions_required: List[ModelPermission]):
        self.permissions_required = permissions_required

    def __call__(self, mocambola: Mocambola = Depends(get_current_mocambola)):
        for permission_required in self.permissions_required:
            print(
                "PermissionChecker, permission_required: %s"
                % permission_required
            )
            for p in [
                *get_role_permissions(mocambola.role),
                *mocambola.permissions,
            ]:
                print("permissions: %s" % p)
            if permission_required not in [
                *get_role_permissions(mocambola.role),
                *mocambola.permissions,
            ]:
                raise HTTPException(
                    status_code=status.HTTP_403_FORBIDDEN,
                    detail="Not enough permissions to access this resource",
                )
        return True
