#!/usr/bin/env python3
"""Grupo de rotas para autenticação."""
from typing import Annotated

from fastapi import Depends
from jose import JWTError, jwt

from baobaxia.constants import ALGORITHM, SECRET_KEY
from baobaxia.exceptions import credentials_exception
from baobaxia.models.auth import TokenData
from baobaxia.root import Baobaxia, bbx
from baobaxia.utils.auth import oauth2_scheme


async def get_current_mocambola(
    token: Annotated[str, Depends(oauth2_scheme)],
    bbx: Annotated[Baobaxia, Depends(bbx)],
):
    try:
        payload = jwt.decode(token, SECRET_KEY, algorithms=[ALGORITHM])
        username: str = payload.get("sub")
        if username is None:
            raise credentials_exception
        token_data = TokenData(username=username)
    except JWTError:
        raise credentials_exception
    mocambola = bbx.get_full_mocambola(username=token_data.username)
    if mocambola is None:
        raise credentials_exception
    return mocambola
