from pathlib import Path

from datalad.tests.utils import ok_file_under_git
from pydantic import BaseModel

from baobaxia.saberes import Balaio, Mocambola, Mucua, SaberesDataStore
from baobaxia.sankofa import Sankofa

from .test_env import BaseBaobaxiaTest, DataTestModel


class TestSankofa(BaseBaobaxiaTest):

    balaio2_name = "rede-mocambos"
    balaio2_smid = "892xa89sajka"
    balaio2_slug = balaio2_name + "_" + balaio2_smid
    balaio2_path = BaseBaobaxiaTest.data_path / balaio2_slug
    balaio2_filepath = balaio2_path / ".baobaxia"
    balaio2_mucua_path = balaio2_path / BaseBaobaxiaTest.mucua_slug
    balaio2_mucua_filepath = balaio2_mucua_path / ".baobaxia"

    balaio2 = Balaio(
        path=Path(balaio2_slug),
        name=balaio2_name,
        smid=balaio2_smid,
        slug=balaio2_slug,
        creator=BaseBaobaxiaTest.mocambola_username,
        created=BaseBaobaxiaTest.default_date,
        last_update=BaseBaobaxiaTest.default_date,
        mucua_padrao=BaseBaobaxiaTest.mucua_slug,
    )

    balaio2_mucua = Mucua(
        path=Path(BaseBaobaxiaTest.mucua_slug),
        name=BaseBaobaxiaTest.mucua_name,
        smid=BaseBaobaxiaTest.mucua_smid,
        slug=BaseBaobaxiaTest.mucua_slug,
        creator=BaseBaobaxiaTest.mocambola_username,
        created=BaseBaobaxiaTest.default_date,
        last_update=BaseBaobaxiaTest.default_date,
        balaio=balaio2_slug,
    )

    content_text = "ABCDEFGHIJKLMNOPQRSTUVWXYZ abcdefghijklmnopqrstuvwxyz"
    content_byte = bytes([100, 24, 255])

    nova_mucua_slug = "abdias-nascimento_LZsgH"
    nova_mucua_remote = "ssh://exu@adbias.mocambos.net:2067/data/bbx/balaios/acervo-patrimonio-red_jNXhc"

    def test_add(self):
        datastore = SaberesDataStore(self.config)

        balaio_dataset = datastore.create_balaio_dataset(
            mocambola=self.mocambola_username
        )
        balaio_dataset.settle(self.balaio2)
        Sankofa.create_balaio(
            balaio=self.balaio2_slug,
            description=self.balaio2_slug,
            config=self.config,
        )

        mucua_dataset = datastore.create_mucua_dataset(
            balaio_smid=self.balaio2_smid, mocambola=self.mocambola_username
        )
        mucua_dataset.settle(self.balaio2_mucua)
        Sankofa.add(
            saberes=[self.balaio2_mucua],
            mocambola=self.mocambola,
            config=self.config,
        )

        tests_fullpath2 = (
            self.data_path
            / self.balaio2_slug
            / self.mucua_slug
            / self.tests_path
        )
        if not tests_fullpath2.exists():
            tests_fullpath2.mkdir()

        dataset = datastore.create_dataset(
            model=DataTestModel,
            balaio_smid=self.balaio_smid,
            mucua_smid=self.mucua_smid,
            mocambola=self.mocambola_username,
        )
        dataset2 = datastore.create_dataset(
            model=DataTestModel,
            balaio_smid=self.balaio2_smid,
            mucua_smid=self.mucua_smid,
            mocambola=self.mocambola_username,
        )

        self.saber_dir = DataTestModel(
            name=self.saber_dir_name,
            path=self.tests_path,
            message=self.saber_dir_message,
            any_int=self.saber_dir_any_int,
        )
        self.saber_dir = dataset.settle(self.saber_dir)

        self.saber_file1 = DataTestModel(
            name=self.saber_file1_name,
            path=self.saber_dir.path,
            message=self.saber_file1_message,
            any_int=self.saber_file1_any_int,
            content=[self.saber_file1_content],
        )
        self.saber_file1 = dataset.settle(self.saber_file1)

        self.saber_file2 = DataTestModel(
            name=self.saber_file2_name,
            path=self.saber_dir.path,
            message=self.saber_file2_message,
            any_int=self.saber_file2_any_int,
        )
        self.saber_file2.content.append(self.saber_file2_content)
        self.saber_file2 = dataset.settle(self.saber_file2)

        saber_file1_content_path = (
            self.config.data_path
            / self.saber_file1.balaio
            / self.saber_file1.mucua
            / self.saber_file1.path
            / self.saber_file1.content[0]
        )
        saber_file2_content_path = (
            self.config.data_path
            / self.saber_file2.balaio
            / self.saber_file2.mucua
            / self.saber_file2.path
            / self.saber_file2.content[0]
        )
        with open(saber_file1_content_path, "w") as ft:
            ft.write(self.content_text)
        with open(saber_file2_content_path, "wb") as fb:
            fb.write(self.content_byte)

        Sankofa.add(
            saberes=[self.saber_dir, self.saber_file1, self.saber_file2],
            mocambola=self.mocambola,
            config=self.config,
        )

        self.saber_dir2 = DataTestModel(
            name=self.saber_dir_name,
            path=self.tests_path,
            message=self.saber_dir_message,
            any_int=self.saber_dir_any_int,
        )
        self.saber_dir2 = dataset2.settle(self.saber_dir2)

        self.saber_file3 = DataTestModel(
            name=self.saber_file1_name,
            path=self.saber_dir2.path,
            message=self.saber_file1_message,
            any_int=self.saber_file1_any_int,
            content=[self.saber_file1_content],
        )
        self.saber_file3 = dataset2.settle(self.saber_file3)

        self.saber_file4 = DataTestModel(
            name=self.saber_file2_name,
            path=self.saber_dir2.path,
            message=self.saber_file2_message,
            any_int=self.saber_file2_any_int,
        )
        self.saber_file4.content.append(self.saber_file2_content)
        self.saber_file4 = dataset2.settle(self.saber_file4)

        saber_file3_content_path = (
            self.config.data_path
            / self.saber_file3.balaio
            / self.saber_file3.mucua
            / self.saber_file3.path
            / self.saber_file3.content[0]
        )
        saber_file4_content_path = (
            self.config.data_path
            / self.saber_file4.balaio
            / self.saber_file4.mucua
            / self.saber_file4.path
            / self.saber_file4.content[0]
        )
        with open(saber_file3_content_path, "w") as ft:
            ft.write(self.content_text)
        with open(saber_file4_content_path, "wb") as fb:
            fb.write(self.content_byte)

        Sankofa.add(
            saberes=[self.saber_dir2, self.saber_file3, self.saber_file4],
            mocambola=self.mocambola,
            config=self.config,
        )

        assert (
            ok_file_under_git(
                self.config.data_path / self.saber_file1.balaio,
                str(
                    self.saber_file1.mucua
                    / self.saber_file1.path
                    / self.config.saber_file_ext
                ),
            )
            == None
        )

        assert (
            ok_file_under_git(
                self.config.data_path / self.saber_file2.balaio,
                str(
                    self.saber_file2.mucua
                    / self.saber_file2.path
                    / self.config.saber_file_ext
                ),
            )
            == None
        )

        assert (
            ok_file_under_git(
                self.config.data_path / self.saber_file3.balaio,
                str(
                    self.saber_file3.mucua
                    / self.saber_file3.path
                    / self.config.saber_file_ext
                ),
            )
            == None
        )

        assert (
            ok_file_under_git(
                self.config.data_path / self.saber_file4.balaio,
                str(
                    self.saber_file4.mucua
                    / self.saber_file4.path
                    / self.config.saber_file_ext
                ),
            )
            == None
        )

        assert (
            ok_file_under_git(
                self.config.data_path / self.saber_file1.balaio,
                str(
                    self.saber_file1.mucua
                    / self.saber_file1.path
                    / self.saber_file1.content[0]
                ),
            )
            == None
        )

        assert (
            ok_file_under_git(
                self.config.data_path / self.saber_file2.balaio,
                str(
                    self.saber_file2.mucua
                    / self.saber_file2.path
                    / self.saber_file2.content[0]
                ),
                annexed=True,
            )
            == None
        )

        assert (
            ok_file_under_git(
                self.config.data_path / self.saber_file3.balaio,
                str(
                    self.saber_file3.mucua
                    / self.saber_file3.path
                    / self.saber_file3.content[0]
                ),
            )
            == None
        )

        assert (
            ok_file_under_git(
                self.config.data_path / self.saber_file4.balaio,
                str(
                    self.saber_file4.mucua
                    / self.saber_file4.path
                    / self.saber_file4.content[0]
                ),
                annexed=True,
            )
            == None
        )

    def test_remove(self):
        datastore = SaberesDataStore(self.config)
        dataset = datastore.create_dataset(
            model=DataTestModel,
            balaio_smid=self.balaio_smid,
            mucua_smid=self.mucua_smid,
            mocambola=self.mocambola_username,
        )
        saber_files = dataset.find_and_collect("tests/*/*")

        sf1 = (
            self.config.data_path
            / saber_files[0].balaio
            / saber_files[0].mucua
            / saber_files[0].path
        )

        assert sf1.exists()

        Sankofa.remove(
            saberes=[saber_files[0]],
            mocambola=self.mocambola,
            config=self.config,
        )

        assert not sf1.exists()

    def test_sync(self):
        Sankofa.sync(balaio_slug=self.balaio_slug, config=self.config)

    def test_mucua_remote_add_and_get(self):

        Sankofa.mucua_remote_add(
            balaio_slug=self.balaio_slug,
            mucua_slug=self.nova_mucua_slug,
            uri=self.nova_mucua_remote,
            config=self.config,
        )

        remotes = Sankofa.mucua_remote_get(
            balaio_slug=self.balaio_slug, config=self.config
        )

        assert self.nova_mucua_remote in remotes

    def test_mucua_remote_del(self):

        Sankofa.mucua_remote_del(
            balaio_slug=self.balaio_slug,
            mucua_slug=self.nova_mucua_slug,
            config=self.config,
        )

        remotes = Sankofa.mucua_remote_get(
            balaio_slug=self.balaio_slug, config=self.config
        )

        assert not remotes

    def test_mucua_group_add_and_get(self):

        novo_territorio = "territorio-taina"

        Sankofa.mucua_group_add(
            balaio_slug=self.balaio_slug,
            mucua_slug=self.mucua_slug,
            group=novo_territorio,
            config=self.config,
        )

        groups = Sankofa.mucua_group_list(
            balaio_slug=self.balaio_slug,
            mucua_slug=self.mucua_slug,
            config=self.config,
        )

        assert novo_territorio in groups

    def test_mucua_group_del(self):

        novo_territorio = "territorio-taina"

        Sankofa.mucua_group_del(
            balaio_slug=self.balaio_slug,
            mucua_slug=self.mucua_slug,
            group=novo_territorio,
            config=self.config,
        )

        groups = Sankofa.mucua_group_list(
            balaio_slug=self.balaio_slug,
            mucua_slug=self.mucua_slug,
            config=self.config,
        )

        assert not groups
