#!/usr/bin/env python3
import shutil
from datetime import timedelta
from pathlib import Path
from tempfile import mktemp

import pytest
from fastapi.testclient import TestClient
from loguru import logger

from baobaxia.api.v2.endpoints.auth import Token
from baobaxia.constants import ACCESS_TOKEN_EXPIRE_MINUTES
from baobaxia.main import app
from baobaxia.root import Baobaxia, bbx
from baobaxia.saberes import SaberesConfig
from baobaxia.sankofa import Sankofa
from baobaxia.utils.auth import create_access_token

# Read-Only Config
base_config = SaberesConfig(
    default_data_path=Path("./tests/fixtures/bbx-base"),
    default_balaio="MZMAcyg",
    default_mucua="52kCSXg",
    default_mucua_name="Nova Mucua",
)


# Copy to a tmp directory and use this config
@pytest.fixture(scope="module")
def bbx_config_rw(tmp_path_factory):
    tmp = tmp_path_factory.mktemp("tests")
    tmp_rw = Path(tmp) / "bbx-base"
    shutil.copytree("./tests/fixtures", tmp, symlinks=True, dirs_exist_ok=True)

    sc = SaberesConfig(
        default_data_path=Path(tmp_rw),
        default_balaio="MZMAcyg",
        default_mucua="52kCSXg",
        default_mucua_name="Nova Mucua",
    )

    Sankofa.create_balaio(
        balaio="balaio-MZMAcyg",
        description="Mais novo balaio do role",
        config=sc,
    )

    return sc


@pytest.fixture()
def baobaxia_base(bbx_config_rw):
    logger.debug("Inicializando mucua fixture")
    bbx = Baobaxia(config=bbx_config_rw)
    return bbx


@pytest.fixture()
def token():
    access_token_expires = timedelta(minutes=ACCESS_TOKEN_EXPIRE_MINUTES)
    access_token = create_access_token(
        data={"sub": "mocambola"}, expires_delta=access_token_expires
    )
    return Token(access_token=access_token, token_type="bearer").access_token


@pytest.fixture()
def client(bbx_config_rw):
    logger.debug("Inicialize cliente http com override da Baobáxia")
    bbx_override = Baobaxia(config=bbx_config_rw)
    app.dependency_overrides[bbx] = bbx_override
    return TestClient(app)
